% Controlador PI
% H(s) = Kp (1 + 1/Tis)
%
% Ta -> tempo de amostragem
% expfi -> potencia de 10 para o inicio da escala log ? i.e. grafico
%          comeca em 10?(expfi)
% expff -> idem para o final da escala log, i.e., 10^(expff)
%
Kp = 2.0
Ti = 0.2
Ta = 0.01
expfi = -1
expff = +2
%
clf % apaga a figura corrente
%
% definicao da faixa de frequencia
w = logspace(expfi,expff,100);
%
% Diagrama de Bode para o PI no dominio s
%
num=[Kp*Ti Kp];
den=[Ti 0];
[mag,phase]=bode(num,den,w);
mag=20*log10(mag);
% modulo
subplot(2,1,1)
semilogx(w,mag,'k-');
xlabel('rad/s');
ylabel('Modulo (Db)');
grid on
hold on
% fase
subplot(2,1,2)
semilogx(w,phase,'k-');
xlabel('rad/s');
ylabel('Fase');
grid on
hold on
% 
% Controlador PI discretizado atraves da Transformacao Bilinear
%
num=[Kp*(Ta+2*Ti) Kp*(Ta-2*Ti)]
den=[2*Ti -2*Ti]
[mag,phase] = dbode(num,den,Ta,w);
mag=20*log10(mag);
% modulo
subplot(2,1,1)
semilogx(w,mag,'r-');
xlabel('rad/s');
ylabel('Modulo (Db)');
grid on
hold on
% fase
subplot(2,1,2)
semilogx(w,phase,'r-');
xlabel('rad/s');
ylabel('Fase');
grid on
hold on
%
% Controlador PI discretizado
% atraves do Casamento de Polos e Zeros
%
Kd=Kp*Ta/(Ti*(1-exp(-Ta/Ti)))
num=[Kd -Kd*exp(-Ta/Ti)]
den=[1 -1]
[mag,phase] = dbode(num,den,Ta,w);
mag=20*log10(mag);
% modulo
subplot(2,1,1)
semilogx(w,mag,'g-.');
xlabel('rad/s');
ylabel('Modulo (Db)');
grid on
hold on
% fase
subplot(2,1,2)
semilogx(w,phase,'g-.');
xlabel('rad/s');
ylabel('Fase');
grid on
hold on
%
% Controlador PI discretizado atraves de Euler Backward-Differences
%
num=[(Kp*Ti+Kp*Ta) -Kp*Ti]
den=[Ti -Ti]
[mag,phase] = dbode(num,den,Ta,w);
mag=20*log10(mag);
% modulo
subplot(2,1,1)
semilogx(w,mag,'y--');
xlabel('rad/s');
ylabel('Modulo (Db)');
grid on
hold on
% fase
subplot(2,1,2)
semilogx(w,phase,'y--');
xlabel('rad/s');
ylabel('Fase');
grid on
hold on
%
% Controlador PI discretizado atraves de
% Euler Forward-Differences
num=[Kp*Ti Kp*(Ta-Ti)]
den=[Ti -Ti]
[mag,phase] = dbode(num,den,Ta,w);
mag=20*log10(mag);
% modulo
subplot(2,1,1)
semilogx(w,mag,'c--');
xlabel('rad/s');
ylabel('Modulo (Db)');
grid on
hold on
% fase
subplot(2,1,2)
semilogx(w,phase,'c--');
xlabel('rad/s');
ylabel('Fase');
grid on
hold on

