%
%  Controlador PI utilizando transformacao bilinear
%  Ultima atualizacao em 29/09/2017
%  Author: NM
%
function [vetorrk,vetoruk,vetoryk] = sistemadecontrolePI
Fa = 20;         % Sampling frequency
Ta = 1/Fa;       % Sampling time
Duration = 50;   % Duracao em segundos
NumberOfTasksToExecute = round(Duration/Ta)

% Create and configure timer object
tm = timer('ExecutionMode','fixedRate', ...            % Run continuously
    'Period',Ta, ...                                   % Period = sampling time
    'TasksToExecute',NumberOfTasksToExecute, ...       % Runs NumberOfTasksToExecute times
    'TimerFcn',@MyTimerFcn, ...                        % Run MyTimerFcn at each timer event
    'StopFcn', @StopEverything);
% Setup da placa de aquisicao
ai=analoginput('nidaq','Dev1');
ao=analogoutput('nidaq','Dev1')
addchannel(ai,0:1);
addchannel(ao,0);
Set(ai,'ChannelSkewMode','Equisample')
ai.ChannelSkew

% inicializacao de variaveis do sistema de conttrole
rk = 0.0;
yk = 0.0;
uk = 0.0;
uk_1 = 0.0;
ek = 0.0;
ek_1 =0.0;

vetorrk = zeros(NumberOfTasksToExecute+1,1);
vetorrk(1) = rk; % instante 0 -> k=1
vetoryk = zeros(NumberOfTasksToExecute+1,1);
vetoryk(1) = yk;
vetoruk = zeros(NumberOfTasksToExecute+1,1);
vetoruk(1) = uk;
k=2;

% Parametros do controlador PI
Kp = 2;
Ti = 0.1;

% Transformacao bilinear
a = Kp + 2*Kp*Ti/Ta;
b = Kp - 2*Kp*Ti/Ta;
c =  2*Ti/Ta;
a1 = a/c;
a2 = b/c;
%a1 = 3.75;
%a2 = -2.25;

% Start the timer
start(tm)
    function MyTimerFcn(obj,event)
    
    % Leitura da Porta A/D - e(k)
    sample=getsample(ai);
    rk = sample(1);
    yk = sample(2);
    
    %calculo do erro e(k)
    ek = rk - yk;
    
    % Calculo do controle
    % uk = u(k-1) + a1*e(k) + a2*e(k-1)
    uk = uk_1 + a1*ek + a2*ek_1;
    
    
    % Controle de Saturacao
    if uk >= 5.0 
       uk = 5.0;
    else if uk <= -5.0
         uk = -5.0;
        end
    end
    
    % Escrita na Porta D/A - u(k)
    putsample(ao,uk);

    % Salva os valores
    vetorrk(k) = rk;
    vetoryk(k) = yk;
    vetoruk(k) = uk;
  
    % Update de variaveis    
    k=k+1  
    uk_1 = uk;
    ek_1 = ek;
    end

    function StopEverything(obj,event)  
    nsamples = length(vetorrk);
    t=0.0:Ta:(nsamples-1)*Ta;
    plot(t,vetorrk,'-',t,vetoruk,'-.',t,vetoryk,'--');
    grid on
    title('Referencia r(k) (-)    /    Esforco de controle u(k) (-.)    /    Saida da planta y(k) (--)')
    xlabel('tempo (s)');
    ylabel('tensao (Volts)');
    mensagem='acabou'
    end

end     % function controlador

% Copyright 2005 The MathWorks, Inc